// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CG-LAB; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CG-LAB.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: intconvex_hull.c,v 1.4 2005/12/02 10:35:34 nmeskini Exp $
// $URL$
//
// Author(s)     : Naceur Meskini
//=========================================================================

#include"stack-c.h"

extern int* convex_hull_2(double* xy,int nb,int* nbind);
extern int* convex_hull_3(double* x,double* y,double* z,int nbpts,int * nbtab,int* nbfacet,int* facetdegree);

int int_convex_hull_2(char* fname)
{

	static int l1, m1, n1, l2, l3;
	static int minlhs=1, maxlhs=2, minrhs=1, maxrhs=1;
	int NbInd;
	int* data = 0;
	int m = 1;
	int n = 1;
	int i = 0;
	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	CreateVar(2, "i", &m, &n, &l2);

	if (!(m1 == 2))
		{
		sciprint("%s: Incompatible inputs \r\n","convex_hull_2");
		Error(999);
		return 0;
		}

	data = convex_hull_2(stk(l1),n1,&NbInd);
	if(!data)return 0;
	CreateVar(3, "i", &m, &NbInd, &l3);
	//	le nombre d'indice vaut NbInd qu'on transmet a la variable #2, istk(l2)
	*istk(l2) = NbInd;
	//	on stock les indices des points extrimaux de l'enveloppe convexe dans la variable de retour #3
	//	istk(l3)
	for(i = 0 ; i < NbInd ; i++)
	{
		*istk(l3+i) = data[i];
	}
	LhsVar(1) = 2;
	LhsVar(2) = 3;
	// liberation de memoire allouee dans la fonction 'data = convex_hull_2(stk(l1),n1,&NbInd)'
	free(data);

	return 0;
}

int int_convex_hull_3(char* fname)
{

	static int l1, m1, n1,m2,n2,m3,n3,l2, l3,l4;
	static int minlhs=0, maxlhs=1, minrhs=3, maxrhs=3;
	int NbInd   =  0;
	int nbfacet =  0 , facetdegree = 0;
	int* data   =  0;
	int i = 0 , j = 0;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;

	GetRhsVar(1, "d", &m1, &n1, &l1);
	GetRhsVar(2, "d", &m2, &n2, &l2);
	GetRhsVar(3, "d", &m3, &n3, &l3);

// 	if (!(m1 == 2))
// 		{
// 		sciprint("%s: Incompatible inputs \r\n");
// 		Error(999);
// 		return 0;
// 		}

	data = convex_hull_3(stk(l1),stk(l2),stk(l3),n1,&NbInd,&nbfacet,&facetdegree);
	if(!data)return 0;
 	CreateVar(4, "i", &nbfacet, &facetdegree, &l4);

//	on stock les indices des points extrimaux de l'enveloppe convexe dans la variable de retour #4
//	istk(l4)
	for(j=0; j< nbfacet; j++)
	{
		for(i =0; i < facetdegree; i++ )
		{
			*istk(l4+j+i*nbfacet)= data[i+facetdegree*j];
		}
	}
 	LhsVar(1) = 4;
	free(data);

	return 0;
}




